@@ -67,7 +67,7 @@ def _check_tf_torch():
 
 
 # Default backend is PyTorch.
-_BACKEND = 'torch'
+_BACKEND = None
 _USER_SET_BACKEND = False
 
 
@@ -90,7 +90,9 @@ def set_backend(backend):
     global _USER_SET_BACKEND, _BACKEND
     # Check whether the user has modified the backend.
     mod = inspect.getmodule(inspect.stack()[1][0])
-    if 'agml.' not in mod.__name__:
+    if mod is None: # IPython shell
+        _USER_SET_BACKEND = True
+    elif 'agml.' not in mod.__name__:
         _USER_SET_BACKEND = True
 
     # If the backend is the same, don't do anything.
@@ -167,9 +169,12 @@ def _convert_image_to_torch(image):
         if image.shape[0] == 1 and image.shape[-1] <= 3 and image.ndim == 4:
             return torch.from_numpy(image).permute(0, 3, 1, 2).float()
         return image
-    if image.shape[0] > image.shape[-1]:
-        return torch.from_numpy(image).permute(2, 0, 1).float()
-    return torch.from_numpy(image)
+    if image.ndim == 3:
+        if image.shape[0] > image.shape[-1]:
+            return torch.from_numpy(image).permute(2, 0, 1).float()
+    elif image.ndim == 2:
+        return torch.from_numpy(image)
+    return torch.from_numpy(image).float()
 
 
 def _postprocess_torch_annotation(image):
@@ -202,10 +207,12 @@ def scalar_unpack(inp):
     return [as_scalar(item) for item in inp]
 
 
-def is_array_like(inp):
+def is_array_like(inp, no_list = False):
     """Determines if an input is a np.ndarray, torch.Tensor, or tf.Tensor."""
     if isinstance(inp, (list, tuple)): # no need to import tensorflow for this
-        return False
+        if no_list:
+            return False
+        return True
     if isinstance(inp, np.ndarray):
         return True
     if isinstance(inp, torch.Tensor):
@@ -289,3 +296,25 @@ def _add_dataset_to_mro(inst, mode):
         if torch_data.Dataset not in inst.__class__.__bases__:
             inst.__class__.__bases__ += (torch_data.Dataset,)
 
+
+def collate_fn_basic(batch):
+    images = torch.stack([i[0] for i in batch], dim = 0)
+    coco = tuple(zip(*[i[1] for i in batch]))
+    return images, coco
+
+
+def collate_fn_efficientdet(batch):
+    """Collates items together into a batch."""
+    images, targets = tuple(zip(*batch))
+    images = torch.stack(images)
+    images = images.float()
+
+    boxes = [target["bboxes"].float() for target in targets]
+    labels = [target["labels"].float() for target in targets]
+    img_size = torch.stack([target["img_size"] for target in targets]).float()
+    img_scale = torch.stack([target["img_scale"] for target in targets]).float()
+
+    annotations = {
+        "bbox": boxes, "cls": labels,
+        "img_size": img_size, "img_scale": img_scale}
+    return images, annotations, targets
