@@ -13,6 +13,8 @@
 # limitations under the License.
 
 from .loader import AgMLDataLoader
+from .image_loader import ImageLoader
+from .point_cloud import PointCloud
 from .public import (
     public_data_sources, source
 )
@@ -21,3 +23,4 @@ from .tools import (
     coco_to_bboxes, convert_bbox_format
 )
 from . import experimental
+from . import exporters
\ No newline at end of file
