@@ -15,6 +15,8 @@
 import os
 import json
 import copy
+import glob
+import fnmatch
 from typing import Union
 from collections.abc import Sequence
 from decimal import getcontext, Decimal
@@ -25,18 +27,22 @@ from agml.framework import AgMLSerializable
 from agml.data.manager import DataManager
 from agml.data.builder import DataBuilder
 from agml.data.metadata import DatasetMetadata, make_metadata
+from agml.data.exporters.yolo import export_yolo
 from agml.utils.logging import log
 from agml.utils.io import get_file_list, get_dir_list
 from agml.utils.data import load_public_sources
 from agml.utils.general import NoArgument, resolve_list_value
 from agml.utils.random import inject_random_state
-from agml.backend.config import data_save_path, synthetic_data_save_path
+from agml.backend.config import (
+    data_save_path, synthetic_data_save_path, SUPER_BASE_DIR
+)
 from agml.backend.experimental import AgMLExperimentalFeatureWrapper
 from agml.backend.tftorch import (
     get_backend, set_backend,
     user_changed_backend, StrictBackendError,
     _add_dataset_to_mro, # noqa
 )
+from agml.viz.general import show_sample
 
 
 class AgMLDataLoaderMeta(type):
@@ -98,15 +104,28 @@ class AgMLDataLoader(AgMLSerializable, metaclass = AgMLDataLoaderMeta):
     -----
     See the methods for examples on how to use an `AgMLDataLoader` effectively.
     """
+    IS_MULTI_DATASET: bool = False
+
     serializable = frozenset((
-        'info', 'builder', 'manager', 'train_data',
-        'val_data', 'test_data', 'is_split', 'meta_properties'))
+        'info', 'builder', 'manager', 'train_data', 'train_content', 'val_data',
+        'val_content', 'test_data', 'test_content', 'is_split', 'meta_properties'))
 
     def __new__(cls, dataset, **kwargs):
         # If a single dataset is passed, then we use the base `AgMLDataLoader`.
         # However, if an iterable of datasets is passed, then we need to
         # dispatch to the subclass `AgMLMultiDatasetLoader` for them.
         if isinstance(dataset, (str, DatasetMetadata)):
+            if '*' in dataset:  # enables wildcard search for datasets
+                valid_datasets = fnmatch.filter(load_public_sources().keys(), dataset)
+                if len(valid_datasets) == 0:
+                    raise ValueError(
+                        f"Wildcard search for dataset '{dataset}' yielded no results.")
+                if len(valid_datasets) == 1:
+                    log(f"Wildcard search for dataset '{dataset}' yielded only "
+                        f"one result. Returning a regular, single-element data loader.")
+                    return super(AgMLDataLoader, cls).__new__(cls)
+                from agml.data.multi_loader import AgMLMultiDatasetLoader
+                return AgMLMultiDatasetLoader(valid_datasets, **kwargs)
             return super(AgMLDataLoader, cls).__new__(cls)
         elif isinstance(dataset, Sequence):
             if len(dataset) == 1:
@@ -150,8 +169,11 @@ class AgMLDataLoader(AgMLSerializable, metaclass = AgMLDataLoaderMeta):
         # If the dataset is split, then the `AgMLDataLoader`s with the
         # split and reduced data are stored as accessible class properties.
         self._train_data = None
+        self._train_content = None
         self._val_data = None
+        self._val_content = None
         self._test_data = None
+        self._test_content = None
         self._is_split = False
 
         # Set the direct access metadata properties like `num_images` and
@@ -206,7 +228,9 @@ class AgMLDataLoader(AgMLSerializable, metaclass = AgMLDataLoaderMeta):
         Parameters
         ----------
         name : str
-            A name for the custom dataset (this can be any valid string).
+            A name for the custom dataset (this can be any valid string). This
+            can also be a path to the dataset (in which case the name will be
+            the base directory inferred from the path).
         dataset_path : str, optional
             A custom path to load the dataset from. If this is not passed,
             we will assume that the dataset is at the traditional path:
@@ -229,6 +253,11 @@ class AgMLDataLoader(AgMLSerializable, metaclass = AgMLDataLoaderMeta):
                              f"a string that is not an existing dataset in "
                              f"the AgML public data source repository.")
 
+        # Check if the `name` is itself the path to the dataset.
+        if os.path.exists(name):
+            dataset_path = name
+            name = os.path.basename(name)
+
         # Locate the path to the dataset.
         if dataset_path is None:
             dataset_path = os.path.abspath(os.path.join(data_save_path(), name))
@@ -295,6 +324,38 @@ class AgMLDataLoader(AgMLSerializable, metaclass = AgMLDataLoaderMeta):
         information which is provided in the `.metadata` directory of the Helios
         generated dataset, allowing it to contain potentially even more info.
         """
+        # Instantiate from a list of datasets.
+        if isinstance(name, (list, tuple)):
+            if dataset_path is None:
+                dataset_path = [None] * len(name)
+            elif isinstance(dataset_path, str):
+                dataset_path = [dataset_path] * len(name)
+            else:
+                if not len(dataset_path) == len(name):
+                    raise ValueError("The number of dataset paths must be "
+                                     "the same as the number of dataset names.")
+            datasets = [cls.helios(n, dataset_path = dp)
+                        for n, dp in zip(name, dataset_path)]
+            return cls.merge(*datasets)
+
+        # Instantiate from a wildcard pattern.
+        if isinstance(name, str) and '*' in name:
+            if dataset_path is None:
+                dataset_path = os.path.abspath(synthetic_data_save_path())
+            elif not os.path.exists(dataset_path):
+                raise NotADirectoryError(
+                    f"Existing directory '{dataset_path}' for dataset of name "
+                    f"{name} not found, pass a custom path if you want to use "
+                    f"a custom dataset path for the dataset.")
+
+            # Get the list of datasets.
+            possible_datasets = glob.glob(os.path.join(dataset_path, name))
+            if len(possible_datasets) == 0:
+                raise ValueError(f"No datasets found for pattern: {name}.")
+            datasets = [cls.helios(os.path.basename(p), dataset_path = dataset_path)
+                        for p in sorted(possible_datasets)]
+            return cls.merge(*datasets)
+
         # Locate the path to the dataset, using synthetic semantics.
         if dataset_path is None:
             dataset_path = os.path.abspath(
@@ -389,6 +450,8 @@ class AgMLDataLoader(AgMLSerializable, metaclass = AgMLDataLoaderMeta):
             indexes = data[indexes].tolist()
         if isinstance(indexes, int):
             indexes = [indexes]
+        if np.isscalar(indexes):
+            indexes = [indexes.item()] # noqa
         for idx in indexes:
             if idx not in range(len(self)):
                 if idx not in [-i for i in range(1, len(self) + 1, 1)]:
@@ -588,7 +651,7 @@ class AgMLDataLoader(AgMLSerializable, metaclass = AgMLDataLoaderMeta):
         if isinstance(self._train_data, AgMLDataLoader):
             return self._train_data
         self._train_data = self._generate_split_loader(
-            self._train_data, split = 'train')
+            self._train_content, split = 'train')
         return self._train_data
 
     @property
@@ -597,7 +660,8 @@ class AgMLDataLoader(AgMLSerializable, metaclass = AgMLDataLoaderMeta):
         if isinstance(self._val_data, AgMLDataLoader):
             return self._val_data
         self._val_data = self._generate_split_loader(
-            self._val_data, split = 'val')
+            self._val_content, split = 'val')
+        self._val_data.eval()
         return self._val_data
 
     @property
@@ -606,7 +670,8 @@ class AgMLDataLoader(AgMLSerializable, metaclass = AgMLDataLoaderMeta):
         if isinstance(self._test_data, AgMLDataLoader):
             return self._test_data
         self._test_data = self._generate_split_loader(
-            self._test_data, split = 'test')
+            self._test_content, split = 'test')
+        self._test_data.eval()
         return self._test_data
 
     def eval(self) -> "AgMLDataLoader":
@@ -944,8 +1009,7 @@ class AgMLDataLoader(AgMLSerializable, metaclass = AgMLDataLoaderMeta):
                     # Re-map the annotation ID.
                     category_ids = annotations['category_id']
                     category_ids[np.where(category_ids == 0)[0]] = 1  # fix
-                    new_ids = np.array([self._map[c]
-                                        for c in category_ids])
+                    new_ids = np.array([self._map[c] for c in category_ids])
                     annotations['category_id'] = new_ids
                     return image, annotations
 
@@ -1139,7 +1203,7 @@ class AgMLDataLoader(AgMLSerializable, metaclass = AgMLDataLoaderMeta):
 
             # Build new `DataBuilder`s and `DataManager`s for the split data.
             for split, content in contents.items():
-                setattr(self, f'_{split}_data', content)
+                setattr(self, f'_{split}_content', content)
 
         # Otherwise, raise an error for an invalid type.
         else:
@@ -1147,6 +1211,124 @@ class AgMLDataLoader(AgMLSerializable, metaclass = AgMLDataLoaderMeta):
                 "Expected either only ints or only floats when generating "
                 f"a data split, got {[type(i) for i in arg_dict.values()]}.")
 
+    def _is_split_generated(self):
+        """Check if a data split has been generated (not necessarily accessed)"""
+        return any(getattr(self, f'_{split}_content')
+                   is not None for split in ['train', 'val', 'test'])
+
+    def save_split(self, name, overwrite = False):
+        """Saves the current split of data to an internal location.
+
+        This method can be used to save the current split of data to an
+        internal file, such that the same split can be later loaded using
+        the `load_split` method (for reproducibility). This method will only
+        save the actual split of data, not any of the transforms or other
+        parameters which have been applied to the loader.
+
+        Parameters
+        ----------
+        name: str
+            The name of the split to save. This name will be used to identify
+            the split when loading it later.
+        overwrite: bool
+            Whether to overwrite an existing split with the same name.
+        """
+        # Ensure that there exist data splits (train/val/test data).
+        if (
+                self._train_content is None
+                and self._val_content is None
+                and self._test_content is None
+        ):
+            raise NotImplementedError("Cannot save a split of data when no "
+                                      "split has been generated.")
+
+        # Get each of the individual splits, and for semantic segmentation/image
+        # classification, remove the full paths and only save the path relative
+        # to the dataset root (so only the file and its directory are saved).
+        splits = {}
+        if self._info.tasks.ml == 'image_classification':
+            for split in ['train', 'val', 'test']:
+                contents = getattr(self, f'_{split}_content')
+                if contents is not None:
+                    contents = {
+                        os.path.relpath(c, self.dataset_root): v for c, v in contents.items()
+                    }
+                splits[split] = contents
+        elif self._info.tasks.ml == 'semantic_segmentation':
+            for split in ['train', 'val', 'test']:
+                contents = getattr(self, f'_{split}_content')
+                if contents is not None:
+                    contents = {
+                        os.path.relpath(c, self.dataset_root):
+                            os.path.relpath(v, self.dataset_root) for c, v in contents.items()
+                    }
+                splits[split] = contents
+
+        # Save the split to the internal location.
+        split_dir = os.path.join(SUPER_BASE_DIR, 'splits', self.name)
+        os.makedirs(split_dir, exist_ok = True)
+        if os.path.exists(os.path.join(split_dir, f'{name}.json')):
+            if not overwrite:
+                raise FileExistsError(f"A split with the name {name} already exists.")
+        with open(os.path.join(split_dir, f'{name}.json'), 'w') as f:
+            json.dump(splits, f)
+
+    def load_split(self, name, **kwargs):
+        """Loads a previously saved split of data.
+
+        This method can be used to load a previously saved split of data
+        if the split was saved using the `save_split` method. This method
+        will only load the actual split of data, not any of the transforms
+        or other parameters which have been applied to the loader. You can
+        use the traditional split accessors (`train_data`, `val_data`, and
+        `test_data`) to access the loaded data.
+
+        You can also load a pre-defined split for the dataset by using its name
+        (any potential such splits can be found in the dataset info, and are
+        derived from the original dataset).
+
+        Parameters
+        ----------
+        name: str
+            The name of the split to load. This name will be used to identify
+            the split to load.
+        """
+        if kwargs.get('manual_split_set', False):
+            splits = kwargs['manual_split_set']
+
+        else:
+            # Ensure that the split exists.
+            split_dir = os.path.join(SUPER_BASE_DIR, 'splits', self.name)
+            if not os.path.exists(os.path.join(split_dir, f'{name}.json')):
+                split_dir = os.path.join(self.dataset_root, '.splits')
+                if not os.path.exists(os.path.join(split_dir, f'{name}.json')):
+                    raise FileNotFoundError(f"Could not find a split with the name {name}.")
+
+            # Load the split from the internal location.
+            with open(os.path.join(split_dir, f'{name}.json'), 'r') as f:
+                splits = json.load(f)
+
+        # Set the split contents.
+        for split, content in splits.items():
+            # If the data is for image classification or semantic segmentation,
+            # then we need to re-construct the full paths to the images.
+            if len(content) > 0:
+                first_item = list(content.items())[0]
+                if not os.path.isabs(first_item[0]):  # backwards compatibility
+                    if self._info.tasks.ml == 'image_classification':
+                        content = {
+                            os.path.join(self.dataset_root, c): v for c, v in content.items()
+                        }
+                    elif self._info.tasks.ml == 'semantic_segmentation':
+                        content = {
+                            os.path.join(self.dataset_root, c): os.path.join(self.dataset_root, v)
+                            for c, v in content.items()
+                        }
+            else:
+                content = None
+
+            setattr(self, f'_{split}_content', content)
+
     def batch(self, batch_size = None):
         """Batches sets of image and annotation data according to a size.
 
@@ -1575,22 +1757,22 @@ class AgMLDataLoader(AgMLSerializable, metaclass = AgMLDataLoaderMeta):
 
         # The `DataLoader` automatically batches objects using its
         # own mechanism, so we remove batching from this DataLoader.
-        batch_size = loader_kwargs.pop(
-            'batch_size', obj._manager._batch_size)
+        batch_size = loader_kwargs.pop('batch_size', obj._manager._batch_size)
         obj.batch(None)
-        shuffle = loader_kwargs.pop(
-            'shuffle', obj._manager._shuffle)
+        shuffle = loader_kwargs.pop('shuffle', obj._manager._shuffle)
 
         # The `collate_fn` for object detection is different because
         # the COCO JSON dictionaries each have different formats. So,
         # we need to replace it with a custom function.
-        collate_fn = loader_kwargs.pop('collate_fn')
+        collate_fn = loader_kwargs.pop('collate_fn', None)
         if obj.task == 'object_detection' and collate_fn is None:
-            def collate_fn(batch):
-                images = torch.stack(
-                    [i[0] for i in batch], dim = 0)
-                coco = tuple(zip(*[i[1] for i in batch]))
-                return images, coco
+            if any('efficientdet' in i.__class__.__name__.lower() for i in
+                   self._manager._transform_manager.get_transform_states()['dual_transform']):
+                from agml.backend.tftorch import collate_fn_efficientdet
+                collate_fn = collate_fn_efficientdet
+            else:
+                from agml.backend.tftorch import collate_fn_basic
+                collate_fn = collate_fn_basic
 
         # Return the DataLoader with a copy of this AgMLDataLoader, so
         # that changes to this will not affect the returned loader.
@@ -1602,6 +1784,62 @@ class AgMLDataLoader(AgMLSerializable, metaclass = AgMLDataLoaderMeta):
             **loader_kwargs
         )
 
+    def export_yolo(self, yolo_path=None):
+        """Exports the contents of the loader to the YOLO format, ready for training.
+
+        This method exports the contents of the loader to the YOLO format, and saves
+        these contents to a file structure that is ready and compatible for training
+        with Ultralytics YOLO models. This method is intended for object detection
+        datasets, and raises an error if used with non-object detection datasets.
+
+        Parameters
+        ----------
+        yolo_path : str
+            The path to save the YOLO-formatted dataset to.
+
+        Returns
+        -------
+        The path to the saved YOLO-formatted dataset.
+        """
+        if self._info.tasks.ml != 'object_detection':
+            raise ValueError("The `export_yolo` method can only be used for "
+                             "object detection tasks.")
+        return export_yolo(self, yolo_path=yolo_path)
+
+    def show_sample(self, image_only = False, no_show = False):
+        """Shows a single data sample from the dataset.
+
+        This method generates a data sample from the dataset with an image and
+        its corresponding annotation (or, if `image_only` is True, then only the
+        image itself). This data sample is then displayed, unless `no_show` is
+        True in which case the processed sample will simply be returned.
+
+        Parameters
+        ----------
+        image_only : optional
+            Whether to show only the image or the image and the annotation.
+        no_show : optional
+            Whether to display the sample or not.
+
+        Returns
+        -------
+        The data sample with/without annotation.
+        """
+        # Get the sample (and take only the first one in a batch if batched).
+        image, annotations = self[self._manager._get_random_index()]
+        if len(image.shape) == 4:
+            image = image[0]
+            annotations = annotations[0]
+
+        # Show the sample.
+        show_sample(self,
+                    image_only = image_only,
+                    no_show = no_show,
+                    sample = (image, annotations))
+
+
+
+
 
 
 
