@@ -12,6 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import os
+import json
 import types
 import collections
 from typing import Union
@@ -22,12 +24,14 @@ import numpy as np
 from agml.framework import AgMLSerializable
 from agml.data.metadata import DatasetMetadata
 from agml.data.loader import AgMLDataLoader
+from agml.data.exporters.yolo import export_yolo
 from agml.utils.general import (
     resolve_list_value, NoArgument
 )
 from agml.utils.random import seed_context, inject_random_state
 from agml.utils.image import consistent_shapes
 from agml.utils.logging import log
+from agml.backend.config import SUPER_BASE_DIR
 from agml.backend.tftorch import (
     get_backend, set_backend,
     user_changed_backend, StrictBackendError,
@@ -60,7 +64,9 @@ class CollectionWrapper(AgMLSerializable):
     def keys(self):
         return self._keys
 
-    def get_attributes(self, attr):
+    def get_attributes(self, attr, include_names=False):
+        if include_names:
+            return {c.name: getattr(c, attr) for c in self._collection}
         return [getattr(c, attr) for c in self._collection]
 
     def call_method(self, method, args = None, kwargs = None):
@@ -92,7 +98,7 @@ class MultiDatasetMetadata(AgMLSerializable):
     Functionally, this class is just a wrapper around multiple
     `DatasetMetadata` objects, and for each of the traditional
     attributes in the original `DatasetMetadata`, returns a
-    dictionary with all of the values for the corresponding
+    dictionary with all the values for the corresponding
     datasets rather than just a single value.
     """
     serializable = frozenset(("names", "metas", "task"))
@@ -126,6 +132,11 @@ class MultiDatasetMetadata(AgMLSerializable):
                              f"for the provided datasets {self._names}.")
         self._task = tasks[0]
 
+    @property
+    def name(self):
+        """A unique identifier for a specific collection of datasets."""
+        return "-".join(self._names)
+
     def __getattr__(self, attr):
         # This is the main functionality of the `DatasetMetadata`
         # class. Rather than re-writing each of the attributes,
@@ -144,7 +155,7 @@ class MultiDatasetMetadata(AgMLSerializable):
             # If it is a method, then it is just printing.
             if isinstance(obj, types.FunctionType):
                 self._metas.call_method(attr)
-                return lambda: None # To act like a function.
+                return lambda: None  # To act like a function.
 
         # Otherwise, raise an error for an invalid argument.
         else:
@@ -209,6 +220,8 @@ class AgMLMultiDatasetLoader(AgMLSerializable):
     `AgMLDataLoader` objects, and draws upon similar functionality
     to the `DataManager` in order to access data from multiple objects.
     """
+    IS_MULTI_DATASET: bool = True
+
     serializable = frozenset(
         ('info', 'loaders', 'loader_accessors', 'class_meta',
          'set_to_keys', 'bounds', 'batch_size', 'shuffle_data',
@@ -300,8 +313,11 @@ class AgMLMultiDatasetLoader(AgMLSerializable):
         # as empty variables (which will be updated if and when it is split).
         self._is_split = False
         self._train_data = None
+        self._train_content = None
         self._val_data = None
+        self._val_content = None
         self._test_data = None
+        self._test_content = None
 
         # Calculate the distribution of the data.
         self._data_distributions = self._info.num_images
@@ -351,7 +367,7 @@ class AgMLMultiDatasetLoader(AgMLSerializable):
         obj._shuffle_data = loaders[0].shuffle_data
         if obj._shuffle_data:
             obj.shuffle(obj._shuffle_data)
-            
+
         # Transform annotations and resizing.
         obj._loaders.apply(
             lambda x: x._manager._train_manager._set_annotation_remap_hook(
@@ -363,8 +379,11 @@ class AgMLMultiDatasetLoader(AgMLSerializable):
         # Finalize parameters.
         obj._is_split = False
         obj._train_data = None
+        obj._train_content = None
         obj._val_data = None
+        obj._val_content = None
         obj._test_data = None
+        obj._test_content = None
         obj._data_distributions = obj._info.num_images
         obj._num_images = sum(obj._info.num_images.values())
 
@@ -385,9 +404,8 @@ class AgMLMultiDatasetLoader(AgMLSerializable):
             indexes = [indexes]
         for idx in indexes:
             if idx not in range(len(self)):
-                raise IndexError(
-                    f"Index {idx} out of range of "
-                    f"AgMLDataLoader length: {len(self)}.")
+                raise IndexError(f"Index {idx} out of range of "
+                                 f"AgMLDataLoader length: {len(self)}.")
         return self._get_item_impl(resolve_list_value(indexes))
 
     def __iter__(self):
@@ -396,7 +414,7 @@ class AgMLMultiDatasetLoader(AgMLSerializable):
 
     def __repr__(self):
         dsp = ", "
-        out = f"<AgMLDataLoader: (datasets=[{dsp.join(self._info.name)}]"
+        out = f"<AgMLDataLoader: (datasets=[{dsp.join(self._info.name.split('-'))}]"
         out += f", task={self.task}"
         out += f") at {hex(id(self))}>"
         return out
@@ -457,14 +475,14 @@ class AgMLMultiDatasetLoader(AgMLSerializable):
             dataset_path = False
         kwargs.update({'dataset_path': dataset_path})
 
-        # Create all of the loaders.
+        # Create all the loaders.
         self._loaders = CollectionWrapper([
             AgMLDataLoader(dataset, **kwargs) for dataset in datasets],
             keys = datasets)
 
     def _adapt_classes(self, cls = None):
         """Adapts the classes in the loader."""
-        # Get all of the unique classes in the loader.
+        # Get all the unique classes in the loader.
         classes = self._info.classes.values()
         class_values = [[o.lower() for o in c] for c in classes]
         class_values = [i for s in class_values for i in s]
@@ -505,6 +523,11 @@ class AgMLMultiDatasetLoader(AgMLSerializable):
         """
         return self._info
 
+    @property
+    def name(self):
+        """The name of the collection of datasets, based on the individual datasets."""
+        return self._info.name
+
     @property
     def task(self):
         """Returns the ML task that this dataset is constructed for."""
@@ -554,8 +577,7 @@ class AgMLMultiDatasetLoader(AgMLSerializable):
                 f"parent loader has not been split.")
 
         # Create a new `CollectionWrapper` around the datasets.
-        new_collection = CollectionWrapper(
-            loaders, keys = [l_.name for l_ in loaders])
+        new_collection = CollectionWrapper(loaders, keys = [l_.name for l_ in loaders])
 
         # Get the state of the current loader and update it
         # with the new collection of loaders. Then, update
@@ -596,11 +618,11 @@ class AgMLMultiDatasetLoader(AgMLSerializable):
         if batch_size is not None:
             new_loader.batch(batch_size = batch_size)
 
-        # Block out all of the splits of the already split
+        # Block out all the splits of the already split
         # loader and set the `_is_split` attribute to True,
         # preventing future splits, and return.
         for attr in ['train', 'val', 'test']:
-            setattr(new_loader, f'_{attr}_data', None)
+            setattr(new_loader, f'_{attr}_content', None)
         new_loader._is_split = True
         return new_loader
 
@@ -620,6 +642,7 @@ class AgMLMultiDatasetLoader(AgMLSerializable):
             return self._val_data
         self._val_data = self._generate_split_loader(
             self._loaders.get_attributes('val_data'), split = 'val')
+        self._val_data.eval()
         return self._val_data
 
     @property
@@ -629,6 +652,7 @@ class AgMLMultiDatasetLoader(AgMLSerializable):
             return self._test_data
         self._test_data = self._generate_split_loader(
             self._loaders.get_attributes('test_data'), split = 'test')
+        self._test_data.eval()
         return self._test_data
 
     def eval(self):
@@ -957,13 +981,104 @@ class AgMLMultiDatasetLoader(AgMLSerializable):
         # Similar to the `AgMLDataLoader` itself, we will not access the
         # `train/val/test_data` parameters instantly, as this will allow
         # any new transforms or other parameters which are applied to the
-        # parent loader to be also applied to all of the child split loaders
+        # parent loader to be also applied to all the child split loaders
         # until they are actually accessed in this overhead multi-dataset
         # loader class. Then, they will be unset and created.
         self._loaders.apply(
             lambda x: x.split(
                 train = train, val = val, test = test, shuffle = shuffle))
 
+        # However, since we are only working off of the accessed data splits
+        # for all the sub-loaders, this wrapper loader would otherwise not retain
+        # the raw split content (e.g., the `_{train/val/test}_content` attribute
+        # for all of the sub-loaders). So, we store this content in the wrapper
+        # loader itself, such that it can be accessed later.
+        self._train_content = self._loaders.get_attributes('_train_content', include_names=True)
+        self._val_content = self._loaders.get_attributes('_val_content', include_names=True)
+        self._test_content = self._loaders.get_attributes('_test_content', include_names=True)
+
+    def _is_split_generated(self):
+        """Check if a data split has been generated (not necessarily accessed)"""
+        return all(i is not None for i in [
+            self._train_content, self._val_content, self._test_content])
+
+    def save_split(self, name, overwrite = False):
+        """Saves the current split of data to an internal location.
+
+        This method can be used to save the current split of data to an
+        internal file, such that the same split can be later loaded using
+        the `load_split` method (for reproducibility). This method will only
+        save the actual split of data, not any of the transforms or other
+        parameters which have been applied to the loader.
+
+        Parameters
+        ----------
+        name: str
+            The name of the split to save. This name will be used to identify
+            the split when loading it later.
+        overwrite: bool
+            Whether to overwrite an existing split with the same name.
+        """
+        # Get the train/val/test splits for each of the sub-loaders.
+        ds_names = self._loaders.get_attributes('name')
+        train_splits = self._loaders.get_attributes('_train_content')
+        val_splits = self._loaders.get_attributes('_val_content')
+        test_splits = self._loaders.get_attributes('_test_content')
+
+        # Check that there is split data.
+        if (
+                all(i is None for i in train_splits)
+                and all(i is None for i in val_splits)
+                and all(i is None for i in test_splits)
+        ):
+            raise ValueError(
+                f"Cannot save a split when no split has been created.")
+
+        # Remap the dictionary such that there is a train/val/test dict for each dataset.
+        split_data = {name: {
+            'train': train_splits[i],
+            'val': val_splits[i],
+            'test': test_splits[i]
+        } for i, name in enumerate(ds_names)}
+
+        # Save the split data.
+        split_dir = os.path.join(SUPER_BASE_DIR, 'splits', self.name)
+        os.makedirs(split_dir, exist_ok = True)
+        if os.path.exists(os.path.join(split_dir, f'{name}.json')) and not overwrite:
+            raise FileExistsError(f"Split with name {name} already exists. "
+                                  f"Set `overwrite = True` to overwrite it.")
+        with open(os.path.join(split_dir, f'{name}.json'), 'w') as f:
+            json.dump(split_data, f)
+
+    def load_split(self, name):
+        """Loads a previously saved split of data.
+
+        This method can be used to load a previously saved split of data
+        if the split was saved using the `save_split` method. This method
+        will only load the actual split of data, not any of the transforms
+        or other parameters which have been applied to the loader. You can
+        use the traditional split accessors (`train_data`, `val_data`, and
+        `test_data`) to access the loaded data.
+
+        Parameters
+        ----------
+        name: str
+            The name of the split to load. This name will be used to identify
+            the split to load.
+        """
+        # Ensure that the split exists.
+        split_dir = os.path.join(SUPER_BASE_DIR, 'splits', self.name)
+        if not os.path.exists(os.path.join(split_dir, f'{name}.json')):
+            raise FileNotFoundError(f"Could not find a split with the name {name}.")
+
+        # Load the split from the internal location.
+        with open(os.path.join(split_dir, f'{name}.json'), 'r') as f:
+            all_splits = json.load(f)
+
+        # Set the split contents.
+        for ds_name, splits in all_splits.items():
+            self._loaders[ds_name].load_split(None, manual_split_set = splits)
+
     def batch(self, batch_size = None):
         """Batches sets of image and annotation data according to a size.
 
@@ -1149,7 +1264,7 @@ class AgMLMultiDatasetLoader(AgMLSerializable):
         - Albumentations transforms are special in that even transforms which
           would normally be passed to `dual_transform` (e.g., they act on the
           input image and the output annotation) can simply be passed to the
-          `transform` argument and they will automatically be applied.
+          `transform` argument, and they will automatically be applied.
         """
         self._loaders.apply(
             lambda x: x._manager.push_transforms(
@@ -1505,6 +1620,87 @@ class AgMLMultiDatasetLoader(AgMLSerializable):
             **loader_kwargs
         )
 
+    def export_tensorflow(self, **loader_kwargs):
+        """Exports the contents of the loader in a native TensorFlow loader.
+
+        This method wraps the contents of this data loader inside of a
+        `tf.data.Dataset`. This method differs from the `export_torch()`
+        method in that there is no need to convert directly to a
+        `torch.utils.data.DataLoader`, rather if this `AgMLDataLoader`
+        inherits from `tf.data.Dataset`, it can just be directly wrapped
+        into a `tf.data.Dataset`.
+
+        The returned `Dataset` is functionally similar to the `AgMLDataLoader`
+        in terms of preprocessing and transforming. You can pass arguments
+        to the `Dataset` instantiation as keyword arguments to this method.
+
+        Note that the `AgMLDataLoader` which this method encloses is instead
+        a copy of the instance the method is run on, so that any changes to
+        the loader afterwards don't affect the exported loader.
+
+        Parameters
+        ----------
+        loader_kwargs : optional
+            A set of keyword arguments for the `tf.data.Dataset`. See the
+            documentation for the loader for more information.
+
+        Returns
+        -------
+        A `tf.data.Dataset` enclosing a copy of this loader.
+        """
+        from agml.backend.tftorch import tf
+        if get_backend() != 'tensorflow':
+            if user_changed_backend():
+                raise StrictBackendError(
+                    change = 'tensorflow', obj = self.export_tensorflow)
+            set_backend('tensorflow')
+
+        # Make a copy of the `AgMLDataLoader` so the following changes
+        # don't affect the original loader, just the new one.
+        obj = self.copy()
+
+        # Convert to a TensorFlow dataset.
+        obj.as_tensorflow_dataset()
+
+        # The `tf.data.Dataset` automatically batches objects using its
+        # own mechanism, so we remove batching from this DataLoader.
+        batch_size = loader_kwargs.pop(
+            'batch_size', obj._batch_size)
+        obj.batch(None)
+        shuffle = loader_kwargs.pop(
+            'shuffle', obj._shuffle_data)
+
+        # Return the DataLoader with a copy of this AgMLDataLoader, so
+        # that changes to this will not affect the returned loader.
+        return tf.data.Dataset.from_generator(
+            obj,
+            output_signature = obj._output_signature,
+            batch_size = batch_size,
+            shuffle = shuffle,
+            **loader_kwargs
+        )
+
+    def export_yolo(self, yolo_path=None):
+        """Exports the contents of the loader to the YOLO format, ready for training.
+
+        This method exports the contents of the loader to the YOLO format, and saves
+        these contents to a file structure that is ready and compatible for training
+        with Ultralytics YOLO models. This method is intended for object detection
+        datasets, and raises an error if used with non-object detection datasets.
+
+        Parameters
+        ----------
+        yolo_path : str
+            The path to save the YOLO-formatted dataset to.
+
+        Returns
+        -------
+        The path to the saved YOLO-formatted dataset.
+        """
+        if self._info.tasks.ml != 'object_detection':
+            raise ValueError("The `export_yolo` method can only be used for "
+                             "object detection tasks.")
+        return export_yolo(self, yolo_path=yolo_path)
 
 
 
